#!/usr/bin/env ruby
require 'tk'

class Button_Label < TkLabel
  def initialize(*args)
    border = 2
    padx = '3m'
    pady = '1m'

    keys = {}
    keys = _symbolkey2str(args.pop) if args[-1].kind_of?(Hash)

    keys['borderwidth'] = border unless keys.key?('borderwidth')
    keys['padx'] = padx unless keys.key?('padx')
    keys['pady'] = pady unless keys.key?('pady')

    keys['relief'] = :raised

    @cmd = keys.delete('command')

    @delay = 0
    @interval = 0
    @timer = TkTimer.new(0){
      @cmd.call if @enter && @press && @cmd
    }
    @delay = keys.delete('repeatdelay') if keys.key?('repeatdelay')
    @interval = keys.delete('repeatinterval') if keys.key?('repeatinterval')

    args << keys

    super(*args)

    @enter = false
    @press = false
    bind('Enter', proc{@enter = true; state(:active)})
    bind('Leave', proc{
           @enter = @press = false; state(:normal); relief(:raised)
         })
    bind('ButtonPress-1', proc{
           relief(:sunken)
           @press = true
           if @delay > 0 && @interval > 0
             @timer.set_interval(@interval)
             @timer.restart(@delay)
           end
         })
    bind('ButtonRelease-1', proc{
           @timer.stop
           relief(:raised)
           @cmd.call if @enter && @press && @cmd
           @enter = @press = false
         })
  end

  def command(cmd = Proc.new)
    @cmd = cmd
    self
  end
  def command=(cmd)
    @cmd = cmd
  end
end

TkButton.new(:text=>'button', 
             :command=>proc{puts "button is pressed!!"}).pack(:fill=>:x)

TkButton.new(:text=>'button (repeat)', 
             :repeatdelay=>500, :repeatinterval=>100, 
             :command=>proc{puts "button is pressed!!"}).pack(:fill=>:x)

Button_Label.new(:text=>'label', 
                 :command=>proc{puts "label is pressed!!"}).pack(:fill=>:x)

Button_Label.new(:text=>'label (repeat)', 
                 :repeatdelay=>500, :repeatinterval=>100, 
                 :command=>proc{puts "label is pressed!!"}).pack(:fill=>:x)

Tk.mainloop
