#!/usr/bin/env ruby
require 'tk'

class Button_Label < TkLabel
  include TkComposite

  def initialize_composite(keys = {})
    border = 2
    padx = '3m'
    pady = '1m'

    @frame.configure(:borderwidth=>border, :relief=>:raised, 
                     :padx=>padx, :pady=>pady, :highlightthickness=>1)

    @label = TkLabel.new(@frame, :borderwidth=>0, :relief=>:flat)
    @label.pack(:ipadx=>1, :ipady=>1, :anchor=>:center)
    @path = @label.path

    @enter = false
    @press = false

    @repeatdelay = 0
    @repeatinterval = 0
    @timer = TkTimer.new(0){
      @cmd.call if @enter && @press && @cmd
    }

    tag = TkBindTag.new
    tag.bind('Enter', proc{|ev|
               @enter = true
               @label.state(:active)
               @frame.background(@label.activebackground)
             })
    tag.bind('Leave', proc{|ev|
               @enter = false
               @frame.relief(:raised)
               @label.anchor(:center)
               @label.state(:normal)
               @frame.background(@label.background)
             })
    tag.bind('ButtonPress-1', proc{|ev|
               @label.anchor(:se)
               @frame.relief(:sunken)
               @press = true
               if @repeatdelay > 0 && @repeatinterval > 0
                 @timer.set_interval(@repeatinterval)
                 @timer.restart(@repeatdelay)
               end
             })
    tag.bind('ButtonRelease-1', proc{|ev|
               @timer.stop
               @label.anchor(:center)
               @frame.relief(:raised)
               @cmd.call if @enter && @press && @cmd
               @press = false
             })

    @frame.bindtags_unshift(tag)
    @label.bindtags_unshift(tag)

    delegate('DEFAULT', @label)
    delegate('relief', @frame)
    delegate('background', @frame, @label)
    delegate('bg', @frame, @label)

    option_methods([:command, :get_command], 
                   :repeatdelay, 
                   :repeatinterval)

    configure keys unless keys.empty?
  end
  private :initialize_composite

  def command(cmd = Proc.new)
    @cmd = cmd
    self
  end
  def command=(cmd)
    @cmd = cmd
  end
  def get_command
    @cmd
  end
  private :get_command

  def repeatdelay(val=nil)
    if val
      @repeatdelay = val
      self
    else
      @repeatdelay
    end
  end
  def repeatdelay=(val)
    repeatdelay(val)
    val
  end

  def repeatinterval(val=nil)
    if val
      @repeatinterval = val
      self
    else
      @repeatinterval
    end
  end
  def repeatinterval=(val)
    repeatinterval(val)
    val
  end
end

TkButton.new(:text=>'button', 
             :command=>proc{puts "button is pressed!!"}).pack(:fill=>:x)

TkButton.new(:text=>'button (repeat)', 
             :repeatdelay=>500, :repeatinterval=>100, 
             :command=>proc{puts "button is pressed!!"}).pack(:fill=>:x)

Button_Label.new(:text=>'label', 
                 :command=>proc{puts "label is pressed!!"}).pack(:fill=>:x)

Button_Label.new(:text=>'label (repeat)', 
                 :repeatdelay=>500, :repeatinterval=>100, 
                 :command=>proc{puts "label is pressed!!"}).pack(:fill=>:x)

Tk.mainloop
