#!/usr/bin/env ruby
require 'tk'

class LabelFrame < TkFrame
  include TkComposite

  Default_value = {
    :borderwidth => 2, 
    :label_padx => 2, 
    :label_offset => 10, 
    :ipad => 4
  }

  def initialize_composite(keys={})
    @frame.configure(:relief=>:flat, :borderwidth=>0, :highlightthickness=>0)

    @padtop = TkFrame.new(@frame, :relief=>:flat, :borderwidth=>0)

    @border = TkFrame.new(@frame, :relief=>:groove, 
                          :borderwidth=>Default_value[:borderwidth], 
                          :highlightthickness=>0)

    @label = TkLabel.new(@frame, :relief=>:flat, :borderwidth=>0, 
                         :highlightthickness=>0, 
                         :padx=>Default_value[:label_padx], :pady=>0)

    @padbot = TkFrame.new(@border, :relief=>:flat, :borderwidth=>0, 
                          :highlightthickness=>0)

    @cframe = TkFrame.new(@border, :relief=>:flat, :borderwidth=>0, 
                          :highlightthickness=>0)
    @path = @cframe.path

    label_height = @label.winfo_reqheight

    @padtop[:height] = label_height / 2 + label_height % 2 + 1
    @padbot[:height] = label_height / 2 - @border.borderwidth

    @padbot.pack(:side=>:top, :fill=>:x)
    @cframe.pack(:fill=>:both, :expand=>true, 
                 :padx=>Default_value[:ipad], :pady=>Default_value[:ipad])

    @padtop.pack(:side=>:top, :fill=>:x)
    @border.pack(:fill=>:both, :expand=>true)

    @offset = Default_value[:label_offset]
    @label.place(:relx=>0.0, :x=>@offset, :anchor=>:nw, :y=>0)

    delegate('DEFAULT', @cframe)

    delegate('text', @label)
    delegate('font', @label)
    delegate('foreground', @label)
    delegate('state', @label)

    delegate('background', @frame, @padtop, @border, @label, @padbot, @cframe)

    delegate('relief', @frame)
    delegate('borderwidth', @frame)

    delegate_alias('frame_relief', 'relief', @border)
    delegate_alias('frame_borderwidth', 'borderwidth', @border)

    delegate_alias('label_relief', 'relief', @label)
    delegate_alias('label_borderwidth', 'borderwidth', @label)

    option_methods(:ipad, :label_offset)

    @label.bind('Configure') { _label_place }
    @border.bind('Configure') { _label_place }

    configure(keys) unless keys.empty?
  end
  private :initialize_composite

  def ipad(val=nil)
    if val
      # set
      @cframe.pack(:padx=>val, :pady=>val)
    else
      # get
      TkPack.info(@cframe)['padx']
    end
  end

  def label_offset(val=nil)
    if val
      @label.place(:x=>val)
      @offset = val
    else
      @offset
    end
  end

  def _label_place
    label_height = @label.winfo_reqheight
    bd = @border.borderwidth
    @padtop.height = label_height / 2 + label_height % 2 + 1
    h = label_height / 2 - bd
    h = 1 if h <= 0
    @padbot.height = h
  end
  private :_label_place
end

f = LabelFrame.new(:text=>'hoge').pack(:fill=>:both,:expand=>true)
TkButton.new(f, :text=>'XXX').pack(:fill=>:x)
TkButton.new(f, :text=>'asdf').pack(:fill=>:x, :pady=>3)
TkButton.new(f, :text=>'MMMMMMM').pack(:fill=>:x)

Tk.after(5000){
  f.configure(:ipad=>0, :label_offset=>5, 
              :text=>'FUGA', :font=>['Times', 14, :bold, :italic], 
              :label_relief=>f.frame_relief, 
              :label_borderwidth=>f.frame_borderwidth)
}

Tk.after(8000){ 
  f.frame_borderwidth(5)
  f.label_borderwidth(5)
  f.background('red')
  f.foreground('white')
}

Tk.mainloop
