#!/usr/bin/env ruby
require 'tk'

class ScrollText < TkText
  include TkComposite

  def initialize_composite(keys={})
    @mode = true
    @scr = TkScrollbar.new(@frame).pack(:side=>:right, :fill=>:y)

    @text = TkText.new(@frame, :wrap=>'char')
    @text.yscrollbar(@scr)
    @text.pack(:side=>:left, :fill=>:both, :expand=>true)

    @path = @text.path

    delegate('DEFAULT', @text)
    delegate('relief', @frame)
    delegate('borderwidth', @frame)
    delegate('background', @frame, @text, @scr)
    delegate('activebackground', @scr)
    delegate('troughcolor', @scr)
    delegate('repeatdelay', @scr)
    delegate('repeatinterval', @scr)

    delegate_alias('textborderwidth', 'borderwidth', @text)
    delegate_alias('textrelief', 'relief', @text)

    option_methods([:scrollbar, :scrollbar?])

    configure(keys) unless keys.empty?

    p '---------------------------'
    p ['reqheight of text', @text.winfo_reqheight]   #########
    p ['reqheight of frame', @frame.winfo_reqheight] #########
    p '---------------------------'
  end
  private :initialize_composite

  def scrollbar(mode)
    @mode = mode
    if @mode
      @scr.pack(:side=>:right, :fill=>:y, :before=>@text)
    else
      @scr.unpack
    end
  end
  def scrollbar?
    @mode
  end
end

pwin = TkPanedwindow.new(:orient=>:vertical).pack(:fill=>:both, :expand=>:true)
pwin.add(ScrollText.new(pwin))                     # ==> very small !!
pwin.add(ScrollText.new(pwin, :scrollbar=>false))  # ==> very small !!

t = TkText.new(pwin)
p ['reqheight of normal text', t.winfo_reqheight]
pwin.add(t)

Tk.mainloop
