#!/usr/bin/env ruby
require 'tk'

# define methods for menu
def open_file(txtwin, fname)
  if txtwin.modified?
    ret = Tk.messageBox(:icon=>'warning',:type=>'okcancel',:default=>'cancel',
                        :message=>'Buffer is modified. Realy erase? ')
    return if ret == 'cancel'
  end

  fpath = Tk.getOpenFile(:filetypes=>[ ['Ruby Scripts', ['.rb', '.rbw']], 
                                       ['ALL Files', '*'] ])
  return if fpath.empty?

  begin
    src = IO.read(fpath)
  rescue => e
    Tk.messageBox(:icon=>'error', :type=>'ok', 
                  :message=>"Fail to read '#{fpath}'.\n#{e.message}")
  end
  
  txtwin.value = src
  txtwin.modified(false)
  txtwin.edit_reset
  txtwin.focus

  txtwin.mark_set('insert', '1.0')

  fname.value = fpath
end

# --------------------------------
def save_file(txtwin, fname)
  return unless txtwin.modified?

  if fname.value == '-'
    ret = Tk.messageBox(:icon=>'warning',:type=>'okcancel',:default=>'ok',
                        :message=>'Filename is not assigned. Save it? ')
    save_as_file(txtwin, fname) if ret == 'ok'
    return
  end

  begin
    fid = open(fname.value, 'w')
  rescue => e
    Tk.messageBox(:icon=>'error', :type=>'ok', 
                  :message=>"Fail to open '#{fname.value}'.\n#{e.message}")
  end

  begin
    fid.print(txtwin.value)
  rescue =>e
    Tk.messageBox(:icon=>'error', :type=>'ok', 
                  :message=>"Fail to write '#{fname.value}'.\n#{e.message}")
  ensure
    fid.close
  end

  txtwin.modified(false)
end

# --------------------------------
def save_as_file(txtwin, fname)
  fpath = Tk.getSaveFile(:filetypes=>[ ['Ruby Scripts', ['.rb', '.rbw']], 
                                       ['ALL Files', '*'] ])
  return if fpath.empty?

  begin
    fid = open(fpath, 'w')
  rescue => e
    Tk.messageBox(:icon=>'error', :type=>'ok', 
                  :message=>"Fail to open '#{fpath}'.\n#{e.message}")
  end

  begin
    fid.print(txtwin.value)
  rescue =>e
    Tk.messageBox(:icon=>'error', :type=>'ok', 
                  :message=>"Fail to write '#{fpath}'.\n#{e.message}")
  ensure
    fid.close
  end

  txtwin.modified(false)
  fname.value = fpath
end

# --------------------------------
def eval_file(txtwin)
  system('ruby', '-e', txtwin.value)
end

# --------------------------------
def quit(txtwin)
  if txtwin.modified?
    ret = Tk.messageBox(:icon=>'warning', :type=>'okcancel', 
                        :default=>'cancel', 
                        :message=>'Buffer is modified. Realy quit? ')
    exit if ret == 'ok'
  else
    exit
  end
end

# --------------------------------
# setup root
root = TkRoot.new(:title=>'Ruby/Tk test window')

# create text frame
f = TkFrame.new(root, :borderwidth=>2, 
                :relief=>:ridge).pack(:fill=>:both, :expand=>true)
txt = TkText.new(f, :undo=>true, :maxundo=>-1, :autoseparators=>true){
  yscrollbar(TkScrollbar.new(f).pack(:side=>:right, :fill=>:y))
  pack(:side=>:left, :fill=>:both, :expand=>true)
}
txt.focus

# create filename frame
fname = TkVariable.new('-')
f = TkFrame.new(root).pack(:fill=>:x)
label = TkLabel.new(f, :textvariable=>fname, 
                    :font=>{:size=>-12, :weight=>:bold}, 
                    :anchor=>'w').pack(:side=>:left, :fill=>:x, :padx=>5)
btn = TkButton.new(f, :text=>'exec', 
                   :command=>proc{
                     eval_file(txt)
                   }).pack(:side=>:right, :padx=>3, :pady=>2)

# create menu
mspec = [
  [ ['File', 0], 
    ['Open', proc{open_file(txt, fname)}, 0], 
    ['Save', proc{save_file(txt, fname)}, 0], 
    ['Save As', proc{save_as_file(txt, fname)}, 5], 
    '---', 
    ['Exec', proc{eval_file(txt)}, 0], 
    '---', 
    ['Quit', proc{quit(txt)}, 0]
  ]
]
root.add_menubar(mspec)

# manage wm_protocol
root.protocol(:WM_DELETE_WINDOW){quit(txt)}

# --------------------------------
# start eventloop
Tk.mainloop
